// Headers are available because we added the pls target
#include <string>
#include <cstdio>
#include <tuple>
#include <array>

#include <pls/dataflow/internal/inputs.h>
#include <pls/dataflow/internal/outputs.h>
#include <pls/dataflow/internal/function_node.h>
#include <pls/dataflow/internal/graph.h>
#include <pls/dataflow/internal/out_port.h>

int main() {
  using namespace pls::dataflow;
  using namespace pls::dataflow::internal;

  // Define
  graph<inputs<int, int>, outputs<int>> graph;

  auto func1 = [](const int &i1, const int &i2, int &o1) {
    std::cout << "Add up " << i1 << " and " << i2 << "..." << std::endl;
    o1 = i1 + i2;
  };
  function_node<inputs<int, int>, outputs<int>, decltype(func1)> node1{func1};

  auto func2 = [](const int &i1, int &o1) {
    std::cout << "Print Result " << i1 << std::endl;
    o1 = i1;
  };
  function_node<inputs<int>, outputs<int>, decltype(func2)> node2{func2};

  // Connect
  graph.input<0>() >> node1.in_port<0>();
  graph.input<1>() >> node1.in_port<1>();

  node1.out_port<0>() >> node2.in_port<0>();

  node2.out_port<0>() >> graph.output<0>();

  // Build
  graph.build();

  // Execute
  graph.run({1, 2});
  graph.run({1, 1});
  graph.run({5, 6});
}
